/* jcifs smb client library in Java
 * Copyright (C) 2002  "Michael B. Allen" <jcifs at samba dot org>
 *					   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.rap;

import java.io.UnsupportedEncodingException;

import jcifs.Config;

public class Buffer {

	public static final String ENCODING =
            Config.getProperty("jcifs.smb.client.codepage",
                    Config.getProperty("jcifs.encoding",
                            System.getProperty("file.encoding")));

    public byte[] buffer;

    public int index;

    public int offset;

    public int start;

    private int base;

    public Buffer(byte[] buffer, int index) {
        this(buffer, index, -1);
    }

    public Buffer(byte[] buffer, int index, int offset) {
        this.buffer = buffer;
        this.index = index;
        this.offset = offset;
        start = index;
        base = offset;
    }

    public int length() {
        return Math.max(index, offset) - start;
    }

    public void pad(int count) {
        for (int i = 0; i < count; i++) buffer[index++] = (byte) 0x00;
    }

    public void skip(int count) {
        index += count;
    }

    public int readShort() {
        return (buffer[index++] & 0xff) + ((buffer[index++] & 0xff) << 8);
    }

    public void writeShort(long value) {
		buffer[index++] = (byte) ((int) value & 0xff);
		buffer[index++] = (byte) ((int) (value >>> 8) & 0xff);
    }

    public int readLong() {
		return (buffer[index++] & 0xff) + ((buffer[index++] & 0xff) << 8) +
				((buffer[index++] & 0xff) << 16) +
                        ((buffer[index++] & 0xff) << 24);
    }

    public void writeLong(long value) {
		buffer[index++] = (byte) ((int) value & 0xff);
		buffer[index++] = (byte) ((int) (value >>> 8) & 0xff);
		buffer[index++] = (byte) ((int) (value >>> 16) & 0xff);
		buffer[index++] = (byte) ((int) (value >>> 24) & 0xff);
    }

    public String readString() {
        return readString(Integer.MAX_VALUE);
    }

    public String readString(int maxLength) {
		int length = 0;
        while (buffer[index + length] != (byte) 0x00) {
            if (length++ > maxLength) throw new IndexOutOfBoundsException();
        }
		try {
            return new String(buffer, index, length, ENCODING);
		} catch (UnsupportedEncodingException ex) {
            throw new IllegalStateException("Encoding \"" + ENCODING +
                    "\" not supported.");
		} finally {
            index += length + 1;
        }
    }

    public void writeString(String string) {
        if (string == null) string = "";
		try {
            byte[] bytes = string.getBytes(ENCODING);
            System.arraycopy(bytes, 0, buffer, index, bytes.length);
            index += bytes.length;
            buffer[index++] = (byte) 0x00;
		} catch (UnsupportedEncodingException ex) {
            throw new IllegalStateException("Encoding \"" + ENCODING +
                    "\" not supported.");
		}
    }

    public String readFixedString(int length) {
        int start = index;
        try {
            return readString(length);
        } finally {
            index = start + length + 1;
        }
    }

    public void writeFixedString(String string, int length) {
        if (string == null) string = "";
        if (string.length() > length) string = string.substring(0, length);
        int pos = index;
        writeString(string);
        pos += length + 1;
        while (index < pos) buffer[index++] = (byte) 0x00;
    }

    public void writeFreeString(String string) {
        if (string == null) {
            writeLong(0);
        } else {
            writeLong(offset);
            try {
                byte[] bytes = string.getBytes(ENCODING);
                System.arraycopy(bytes, 0, buffer, offset, bytes.length);
                offset += bytes.length;
                buffer[offset++] = (byte) 0x00;
            } catch (UnsupportedEncodingException ex) {
                throw new IllegalStateException("Encoding \"" + ENCODING +
                        "\" not supported.");
            }
        }
    }

    public String readFreeString() {
        int pointer = readLong() & 0xffff;
        if (pointer == 0) return null;
        pointer += base;
		int length;
        for (length = 0; buffer[pointer + length] != (byte) 0x00; length++);
		try {
            return new String(buffer, pointer, length, ENCODING);
		} catch (UnsupportedEncodingException ex) {
            throw new IllegalStateException("Encoding \"" + ENCODING +
                    "\" not supported.");
		}
    }

}
